<?php

namespace Drupal\wcm_simplesamlphp_auth;

use Drupal\Core\Url;

/**
 * Functions for verifying and refreshing simpleSAMLPHP metadata.
 *
 *  $metadataFile - name of the metadata file.
 *  $paramFilePath - relative path to the parameters.php file.
 *  $maxMetadataAge - maximum age in seconds of the metadata file.
 */
class Metarefresh {

  private $metadataFile = '/files/private/simplesamlphp/metadata/saml20-idp-remote.php';
  private $parametersFile = '/files/private/simplesamlphp/config/parameters.php';
  private $minTimeToExpire = 43200;

  /**
   * Check metadata expiration and refresh if needed.
   */
  public function refreshMetadata() {
    // Refresh flag.
    $refresh = FALSE;
    // Log time to expire.
    $timeToExpire = $this->getTimeToMetadataExpiration();
    $msg = sprintf(
      'SimpleSAMLphp metadata expires in %s seconds.',
      $timeToExpire
    );
    \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    // If time to expire is less than _minTimeToExpire, refresh.
    if ($timeToExpire < $this->minTimeToExpire) {
      $this->requestMetadataRefresh();
    }
  }

  /**
   * Hit the metadata refresh URL.
   */
  private function requestMetadataRefresh() {
    // Default to HTTPS
    $httpSecure = TRUE;
    $httpPrefix = 'https';
    // Check if we're allowing http 
    $allowHttpRefresh = $this->getSimplesamlphpParameter('allowHttpRefresh');
    if ($allowHttpRefresh) {
      // Log the override setting.
      $msg = sprintf(
        'Simplesaml metadata refresh setting allowHttpRefresh is enabled.'
      );
    \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
      // Determine HTTPS from server vars
      if (isset($_SERVER['HTTPS']) && !$_SERVER['HTTPS']) {
        $httpSecure = FALSE;
        $httpPrefix = 'http';
      }
    }
    // Default to cronURL param.
    $cronModuleUrl = $this->getSimplesamlphpParameter('cronModuleURLPath');
    $cronModuleUrlSource = "parameter";
    // No cronURL parameter. Construct URL.
    if (!strlen($cronModuleUrl)) {
      // Get the baseURLpath parameter
      $baseURL = $this->getSimplesamlphpParameter('baseURLPath');
      $cronModuleUrl = trim($baseURL . '/simplesaml/module.php/cron/cron.php');
      $cronModuleUrlSource = "constructed";
    } 
    // Rewrite parameter if it was given an http URL and http is not allowed.
    elseif (!$allowHttpRefresh && substr($cronModuleUrl, 0, 5) != 'https') {
      $oldCronModuleUrl = $cronModuleUrl;
      $cronModuleUrl = str_replace('http', 'https', $cronModuleUrl);
      // Log the corrected url.
      $msg = sprintf(
        'Simplesaml metadata refresh requires https. Rewriting cronModuleUrl parameter from %s to %s',
        $oldCronModuleUrl,
        $cronModuleUrl,
      );
    \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);

    }
    // Make sure a refresh URL was found.
    if (!strlen(trim($cronModuleUrl))) {
      // Log error and exit.
      $msg = "SimplesamlPHP metadata refresh failed. Could not determine a metadata refresh URL.";
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
      exit;
    }
    // Get cronkey parameter.
    $cronKey = $this->getSimplesamlphpParameter('cronKey');
    // Create URL options array.
    $urlOptions = [
      'query' => [
        'key' => $cronKey,
        'tag' => 'hourly',
      ],
      'absolute' => TRUE,
      'https' => $httpSecure,
    ];
    // Construct refresh URL.
    $metaRefreshUrl = Url::fromUri($cronModuleUrl, $urlOptions)->toString();
    // Log the source and cron url.
    $msg = sprintf(
      'Using %s value for metaRefreshUrl: %s',
      $cronModuleUrlSource,
      $metaRefreshUrl
    );
    \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    // Use curl to hit the cron endpoint and kick metarefresh.
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $metaRefreshUrl);
    $success = curl_exec($ch);
    // Get the return code.
    $curlReturnCode =  curl_getinfo($ch, CURLINFO_HTTP_CODE);
    // Check if curl was successful. 
    if ($success === FALSE) {
      $msg = sprintf(
        'Simplesamlphp metarefresh failed with http code %s',
        $curlReturnCode
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
    }
    // Report success when we get 200.
    elseif ($curlReturnCode == 200) {
      $msg = sprintf(
        'Simplesamlphp metarefresh succeeded. Returned http %s',
        $curlReturnCode
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->notice($msg);
    }
    // Report anything other than 200 as an error.
    else {
      $msg = sprintf(
        'Simplesamlphp metarefresh result unclear. Returned http %s',
        $curlReturnCode
      );
      \Drupal::logger('wcm_simplesamlphp_auth')->error($msg);
    }
    curl_close($ch);
  }

  /**
   * Returns the current metadata expiration.
   *
   * @return int
   *   Epoch time expiration value.
   */
  private function getTimeToMetadataExpiration() {
    // Default to expired.
    $timeToExpire = 0;
    // File exists. Get expire value.
    if (file_exists($this->metadataFile)) {
      include_once $this->metadataFile;
      $expires = $metadata['urn:mace:incommon:osu.edu']['expire'];
      $timeToExpire = $expires - time();
    }
    // Return time to expire.
    return $timeToExpire;
  }

  /**
   * Returns the current value for the passed parameter.
   *
   * @param string $param
   *   String name of the parameter to fetch.
   *
   * @return string
   *   Value of the parameter.
   */
  private function getSimplesamlphpParameter($param) {
    $paramValue = "";
    $saml_parameters = [];
    include $this->parametersFile;
    // Return requested value from parameter array.
    if (array_key_exists($param, $saml_parameters)) {
      $paramValue = $saml_parameters[$param];
    }
    return $paramValue;
  }

}
